// Copyright (C) 2024 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

#include <QJsonValue>
#include <QTimer>
#include <QWebSocket>

class DesignStudio : public QObject
{
    Q_OBJECT
public:
    DesignStudio(QWebSocket *socket, const QString &deviceID, QObject *parent = nullptr);

    // Getters
    QString ipv4Addr() const;
    QString id() const;

    // Send data
    void sendDeviceInfo();
    void sendProjectStarting();
    void sendProjectRunning();
    void sendProjectStopped();
    void sendProjectLogs(const QString &logs);

private:
    // Network
    QScopedPointer<QWebSocket> m_socket;
    bool m_socketWasConnected;

    QTimer m_pingTimer;
    QTimer m_pongTimer;

    // DS data
    QString m_designStudioID;
    QString m_deviceID;

    // project
    QByteArray m_projectData;
    int m_incomingProjectSize;
    int m_lastPercentage;
    QTime m_projectNotificationTime;
    QTimer m_speedCalculator;
    QTimer m_projectStallTimer;

    // DS comm
    void sendData(const QLatin1String &dataType, const QJsonValue &data = QJsonValue());

    // Internal
    void initPingPong();
    void startPingPong();
    void stopPingPong();
    void initSocket();

private slots:
    void processTextMessage(const QString &data);
    void processBinaryMessage(const QByteArray &data);

signals:
    // socket signals
    void disconnected(const QString &id);
    void connected(const QString &id);

    // DS signals
    void idReceived(const QString &id, const QString &ipv4Addr);
    void projectVersionMismatch(const QString &version, const QString &got);
    void projectIncoming(const QString &id, const int projectSize);
    void projectIncomingProgress(const QString &id, const int percentage);
    void projectReceived(const QString &id, const QByteArray &data);
    void projectStopRequested(const QString &id);
};
